@@ -60,26 +60,14 @@ G_DEFINE_TYPE (EvSidebar, ev_sidebar, GTK_TYPE_VBOX)
 #define EV_SIDEBAR_GET_PRIVATE(object) \
 		(G_TYPE_INSTANCE_GET_PRIVATE ((object), EV_TYPE_SIDEBAR, EvSidebarPrivate))
 
-static void
-ev_sidebar_destroy (GtkObject *object)
-{
-	(* GTK_OBJECT_CLASS (ev_sidebar_parent_class)->destroy) (object);
-}
-
 static void
 ev_sidebar_class_init (EvSidebarClass *ev_sidebar_class)
 {
 	GObjectClass *g_object_class;
-	GtkWidgetClass *widget_class;
-	GtkObjectClass *gtk_object_klass;
  
 	g_object_class = G_OBJECT_CLASS (ev_sidebar_class);
-	widget_class = GTK_WIDGET_CLASS (ev_sidebar_class);
-	gtk_object_klass = GTK_OBJECT_CLASS (ev_sidebar_class);
 	   
 	g_type_class_add_private (g_object_class, sizeof (EvSidebarPrivate));
-	   
-	gtk_object_klass->destroy = ev_sidebar_destroy;
 
 	ev_sidebar_table_signals[CLOSED] =
 		g_signal_new ("closed",
