@@ -444,7 +444,9 @@ totem_uri_get_subtitle_uri (const char *uri)
 	GFile *file;
 	char *subtitle;
 
-	if (g_str_has_prefix (uri, "http") != FALSE)
+	if (g_str_has_prefix (uri, "http") != FALSE ||
+	    g_str_has_prefix (uri, "rtsp") != FALSE ||
+	    g_str_has_prefix (uri, "rtmp") != FALSE)
 		return NULL;
 
 	/* Has the user specified a subtitle file manually? */
@@ -579,7 +581,7 @@ char *
 totem_add_subtitle (GtkWindow *parent, const char *uri)
 {
 	GtkWidget *fs;
-	GConfClient *conf;
+	GSettings *settings;
 	char *new_path;
 	char *subtitle = NULL;
 	gboolean folder_set;
@@ -596,7 +598,7 @@ totem_add_subtitle (GtkWindow *parent, const char *uri)
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_subs);
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (fs), filter_subs);
 
-	conf = gconf_client_get_default ();
+	settings = g_settings_new (TOTEM_GSETTINGS_SCHEMA);
 	folder_set = FALSE;
 
 	/* Add the subtitles cache dir as a shortcut */
@@ -608,10 +610,9 @@ totem_add_subtitle (GtkWindow *parent, const char *uri)
 	g_free (new_path);
 
 	/* Add the last open path as a shortcut */
-	new_path = gconf_client_get_string (conf, "/apps/totem/open_path", NULL);
-	if (new_path != NULL && *new_path != '\0') {
+	new_path = g_settings_get_string (settings, "open-uri");
+	if (*new_path != '\0')
 		gtk_file_chooser_add_shortcut_folder_uri (GTK_FILE_CHOOSER (fs), new_path, NULL);
-	}
 	g_free (new_path);
 
 	/* Try to set the passed path as the current folder */
@@ -633,6 +634,8 @@ totem_add_subtitle (GtkWindow *parent, const char *uri)
 	}
 
 	gtk_widget_destroy (fs);
+	g_object_unref (settings);
+
 	return subtitle;
 }
 
@@ -643,7 +646,7 @@ totem_add_files (GtkWindow *parent, const char *path)
 	int response;
 	GSList *filenames;
 	char *mrl, *new_path;
-	GConfClient *conf;
+	GSettings *settings;
 	gboolean set_folder;
 
 	fs = gtk_file_chooser_dialog_new (_("Select Movies or Playlists"),
@@ -661,14 +664,14 @@ totem_add_files (GtkWindow *parent, const char *path)
 	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fs), TRUE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
 
-	conf = gconf_client_get_default ();
+	settings = g_settings_new (TOTEM_GSETTINGS_SCHEMA);
 	set_folder = TRUE;
 	if (path != NULL) {
 		set_folder = gtk_file_chooser_set_current_folder_uri
 			(GTK_FILE_CHOOSER (fs), path);
 	} else {
-		new_path = gconf_client_get_string (conf, "/apps/totem/open_path", NULL);
-		if (new_path != NULL && *new_path != '\0') {
+		new_path = g_settings_get_string (settings, "open-uri");
+		if (*new_path != '\0') {
 			set_folder = gtk_file_chooser_set_current_folder_uri
 				(GTK_FILE_CHOOSER (fs), new_path);
 		}
@@ -686,14 +689,14 @@ totem_add_files (GtkWindow *parent, const char *path)
 
 	if (response != GTK_RESPONSE_ACCEPT) {
 		gtk_widget_destroy (fs);
-		g_object_unref (conf);
+		g_object_unref (settings);
 		return NULL;
 	}
 
 	filenames = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (fs));
 	if (filenames == NULL) {
 		gtk_widget_destroy (fs);
-		g_object_unref (conf);
+		g_object_unref (settings);
 		return NULL;
 	}
 	gtk_widget_destroy (fs);
@@ -701,12 +704,11 @@ totem_add_files (GtkWindow *parent, const char *path)
 	mrl = filenames->data;
 	if (mrl != NULL) {
 		new_path = g_path_get_dirname (mrl);
-		gconf_client_set_string (conf, "/apps/totem/open_path",
-				new_path, NULL);
+		g_settings_set_string (settings, "open-uri", new_path);
 		g_free (new_path);
 	}
 
-	g_object_unref (conf);
+	g_object_unref (settings);
 
 	return filenames;
 }
